@@ -32,7 +32,7 @@ class WISEDataDESYCluster(WiseDataByVisit):
         self._cluster_queue = queue.Queue()
 
     def get_sample_photometric_data(self, max_nTAPjobs=8, perc=1, tables=None, chunks=None,
-                                    cluster_jobs_per_chunk=100, wait=5, remove_chunks=True):
+                                    cluster_jobs_per_chunk=100, wait=5, remove_chunks=True, query_type='positional'):
         """
         An alternative to `get_photometric_data()` that uses the DESY cluster and is optimised for large datasets.
 
@@ -53,7 +53,7 @@ class WISEDataDESYCluster(WiseDataByVisit):
         """
 
         mag = True
-        flux = False
+        flux = True
 
         if tables is None:
             tables = [
@@ -64,6 +64,9 @@ class WISEDataDESYCluster(WiseDataByVisit):
         if chunks is None:
             chunks = list(range(round(int(self.n_chunks * perc))))
 
+        if query_type not in self.query_types:
+            raise ValueError(f"Unknown query type {query_type}! Choose one of {self.query_types}")
+
         service = 'tap'
         self.queue = queue.Queue()
         tables = np.atleast_1d(tables)
@@ -95,7 +98,7 @@ class WISEDataDESYCluster(WiseDataByVisit):
         logger.debug(f'started {len(tap_threads)} TAP threads and {len(cluster_threads)} cluster threads.')
 
         for c in chunks:
-            self._tap_queue.put((tables, c, wait, mag, flux, cluster_time))
+            self._tap_queue.put((tables, c, wait, mag, flux, cluster_time, query_type))
 
         logger.debug(f'added {self._tap_queue.qsize()} tasks to tap queue')
         self._tap_queue.join()
@@ -103,18 +106,17 @@ class WISEDataDESYCluster(WiseDataByVisit):
         self._cluster_queue.join()
         logger.debug('cluster done')
 
-
         self._combine_lcs(service=service, overwrite=True, remove=remove_chunks)
         self._combine_metadata(service=service, overwrite=True, remove=remove_chunks)
 
     def _tap_thread(self):
         logger.debug('started tap thread')
         while True:
-            tables, chunk, wait, mag, flux, cluster_time = self._tap_queue.get(block=True)
+            tables, chunk, wait, mag, flux, cluster_time, query_type = self._tap_queue.get(block=True)
             logger.debug(f'querying IRSA for chunk {chunk}')
 
             for t in tables:
-                self._submit_job_to_TAP(chunk, t, mag, flux)
+                self._submit_job_to_TAP(chunk, t, mag, flux, query_type)
                 logger.info(f'wating for {wait} hours')
                 time.sleep(wait * 3600)
                 self._thread_wait_and_get_results(t, chunk)
@@ -151,10 +153,12 @@ class WISEDataDESYCluster(WiseDataByVisit):
     def _qstat_output(qstat_command):
         """return the output of the qstat_command"""
         # start a subprocess to query the cluster
-        process = subprocess.Popen(qstat_command, stdout=subprocess.PIPE, shell=True)
-        # read the output
-        tmp = process.stdout.read().decode()
-        return str(tmp)
+        with subprocess.Popen(qstat_command, stdout=subprocess.PIPE, shell=True) as process:
+            # read the output
+            tmp = process.stdout.read().decode()
+            process.terminate()
+            msg = str(tmp)
+        return msg
 
     @staticmethod
     def _get_ids(qstat_command):
@@ -367,8 +371,10 @@ class WISEDataDESYCluster(WiseDataByVisit):
 
         self._make_cluster_script(cluster_h, cluster_ram, tables, service)
 
-        process = subprocess.Popen(submit_cmd, stdout=subprocess.PIPE, shell=True)
-        msg = process.stdout.read().decode()
+        with subprocess.Popen(submit_cmd, stdout=subprocess.PIPE, shell=True) as process:
+            msg = process.stdout.read().decode()
+            process.terminate()
+
         logger.info(str(msg))
         job_id = int(str(msg).split('job-array')[1].split('.')[0])
         logger.info(f"Running on cluster with ID {job_id}")
