@@ -1,4 +1,4 @@
-import os, subprocess, copy, json, argparse, tqdm, time, threading, math, queue, pickle, getpass, requests, abc
+import os, subprocess, copy, json, tqdm, time, threading, queue, requests, abc
 import multiprocessing as mp
 import pandas as pd
 import numpy as np
@@ -36,6 +36,8 @@ class WISEDataBase(abc.ABC):
     running_tap_phases = ["QUEUED", "EXECUTING", "RUN"]
     done_tap_phases = {"COMPLETED", "ABORTED", "ERROR"}
 
+    query_types = ['positional', 'by_allwise_id']
+
 
     table_names = pd.DataFrame([
         ('AllWISE Multiepoch Photometry Table', 'allwise_p3as_mep'),
@@ -53,10 +55,6 @@ class WISEDataBase(abc.ABC):
     error_key_ext = "_error"
     band_plot_colors = {'W1': 'r', 'W2': 'b'}
 
-    mean_key = '_mean'
-    rms_key = '_rms'
-    upper_limit_key = '_ul'
-
     photometry_table_keymap = {
         'AllWISE Multiepoch Photometry Table': {
             'flux': {
@@ -89,6 +87,7 @@ class WISEDataBase(abc.ABC):
     }
 
     # zero points come from https://wise2.ipac.caltech.edu/docs/release/allsky/expsup/sec4_4h.html#conv2flux
+    # published in Jarret et al. (2011): https://ui.adsabs.harvard.edu/abs/2011ApJ...735..112J/abstract
     magnitude_zeropoints = {
         'F_nu': {
             'W1': 309.54 * u.Jy,
@@ -97,9 +96,18 @@ class WISEDataBase(abc.ABC):
         'Fstar_nu': {
             'W1': 306.682 * u.Jy,
             'W2': 170.663 * u.Jy
+        },
+        'Mag': {
+            'W1': 20.752,
+            'W2': 19.596
         }
     }
 
+    aperture_corrections = {
+        'W1': 0.222,
+        'W2': 0.280
+    }
+
     _this_dir = os.path.abspath(os.path.dirname(__file__))
     magnitude_zeropoints_corrections = ascii.read(f'{_this_dir}/wise_flux_conversion_correction.dat',
                                                   delimiter='\t').to_pandas()
@@ -179,6 +187,10 @@ class WISEDataBase(abc.ABC):
         self.tap_jobs = None
         self.queue = queue.Queue()
         self.clear_unbinned_photometry_when_binning = False
+        self._cached_final_products = {
+            'lightcurves': dict(),
+            'metadata': dict()
+        }
 
         self._tap_wise_id_key = 'wise_id'
         self._tap_orig_id_key = 'orig_id'
@@ -225,10 +237,10 @@ class WISEDataBase(abc.ABC):
             logger.warning("No parent sample given! Can not calculate dec interval masks!")
 
     def _get_chunk_number(self, wise_id=None, parent_sample_index=None):
-        if (not wise_id) and (not parent_sample_index):
+        if isinstance(wise_id, type(None)) and isinstance(parent_sample_index, type(None)):
             raise Exception
 
-        if wise_id:
+        if not isinstance(wise_id, type(None)):
             parent_sample_index = np.where(self.parent_sample.df[self.parent_wise_source_id_key] == int(wise_id))[0]
             logger.debug(f"wise ID {wise_id} at index {parent_sample_index}")
 
@@ -278,30 +290,12 @@ class WISEDataBase(abc.ABC):
         :return:
         """
 
+        logger.info(f'matching all chunks to {table_name}')
         for i in range(self.n_chunks):
             self._match_single_chunk(i, table_name)
 
         _dupe_mask = self._get_dubplicated_wise_id_mask()
 
-        # vvvvvvvvvvvv                        THIS IS DEAD CODE                          vvvvvvvvvvvv #
-        rematch = False
-        rematch_to_neowise = False
-        if np.any(_dupe_mask) and rematch:
-            self._rematch_duplicates(table_name, _dupe_mask, filext="_rematch1")
-
-            _inf_mask = ~(self.parent_sample.df[self.parent_sample_wise_skysep_key] < np.inf)
-            if np.any(_inf_mask) and rematch_to_neowise:
-                logger.info(f"Still {len(self.parent_sample.df[_inf_mask])} entries without match."
-                            f"Looking in NoeWISE Photometry")
-                for mi in range(self.n_chunks):
-                    m = self.chunk_map == mi
-                    _interval_inf_mask = _inf_mask & m
-                    if np.any(_interval_inf_mask):
-                        self._rematch_duplicates(table_name='NEOWISE-R Single Exposure (L1b) Source Table',
-                                                 mask=_interval_inf_mask,
-                                                 filext=f"_rematch2_c{mi}")
-        # ^^^^^^^^^^^^^^                      THIS IS DEAD CODE                         ^^^^^^^^^^^^^^ #
-
         self._no_allwise_source = self.parent_sample.df[self.parent_sample_wise_skysep_key] == np.inf
         if np.any(self._no_allwise_source):
             logger.warning(f"{len(self.parent_sample.df[self._no_allwise_source])} of {len(self.parent_sample.df)} "
@@ -362,7 +356,7 @@ class WISEDataBase(abc.ABC):
             logger.info(out_msg.decode())
         if err_msg:
             logger.error(err_msg.decode())
-
+        process.terminate()
         if os.path.isfile(out_file):
             return 1
         else:
@@ -468,58 +462,6 @@ class WISEDataBase(abc.ABC):
             logger.info(f"{_N_dupe} duplicated entries in parent sample")
         return _dupe_mask
 
-    def _rematch_duplicates(self, table_name, mask=None, filext=""):
-        if mask is None:
-            mask = self._get_dubplicated_wise_id_mask()
-
-        for k, default in self.parent_sample_default_entries.items():
-            self.parent_sample.df.loc[mask, k] = default
-
-        _dupe_infile = os.path.join(self.cache_dir, f"parent_sample_duplicated{filext}.xml")
-        _dupe_output_file = os.path.join(self.cache_dir, f"parent_sample_duplicated{filext}.tbl")
-        _gator_res = self._match_to_wise(
-            in_filename=_dupe_infile,
-            out_filename=_dupe_output_file,
-            mask=mask,
-            table_name=table_name,
-            one_to_one=False
-        ).to_pandas()
-
-        if len(_gator_res) == 0:
-            logger.debug('No additional matches found')
-            return
-
-        _dupe_inds = list(self.parent_sample.df[mask].index)
-        for in_id in tqdm.tqdm(_dupe_inds, desc='finding match for duplicates'):
-            _m = _gator_res.index_01 == in_id
-            _sorted_sel = _gator_res[_m].sort_values('dist_x')
-            for _, r in _sorted_sel.iterrows():
-                _wise_id = r['cntr' if 'allwise' in self.get_db_name(table_name) else 'allwise_cntr']
-                _skysep = r['dist_x']
-                __m = self.parent_sample.df[self.parent_wise_source_id_key] == _wise_id
-
-                if not np.any(__m):
-                    self.parent_sample.df.loc[in_id, self.parent_sample_wise_skysep_key] = _skysep
-                    self.parent_sample.df.loc[in_id, self.parent_wise_source_id_key] = _wise_id
-                    break
-
-                else:
-                    __msep = self.parent_sample.df[self.parent_sample_wise_skysep_key][__m] > _skysep
-                    if np.any(__msep):
-                        columns = ['ra', 'dec', self.parent_sample_wise_skysep_key, self.parent_wise_source_id_key]
-                        txt = f"WISE ID: {_wise_id} with skysep {_skysep}:\n" \
-                              f"{self.parent_sample.df.loc[in_id].to_string()}" \
-                              f" \n" \
-                              f"{self.parent_sample.df.loc[__m].to_string(columns=columns)}"
-                        if len(self.parent_sample.df[self.parent_wise_source_id_key][__m][__msep]) == 1:
-                            logger.warning(txt)
-                            self.parent_sample.df.loc[in_id, self.parent_sample_wise_skysep_key] = _skysep
-                            self.parent_sample.df.loc[in_id, self.parent_wise_source_id_key] = _wise_id
-                            for k in [self.parent_wise_source_id_key, self.parent_sample_wise_skysep_key]:
-                                self.parent_sample.df.loc[np.where(__m)[0][__msep], k] = self.parent_sample_default_entries[k]
-                        else:
-                            raise Exception(txt)
-
     ###################################################
     # END MATCH PARENT SAMPLE TO WISE SOURCES         #
     ###########################################################################################################
@@ -529,7 +471,7 @@ class WISEDataBase(abc.ABC):
     ###################################
 
     def get_photometric_data(self, tables=None, perc=1, wait=0, service=None, nthreads=100,
-                             chunks=None, overwrite=False, remove_chunks=False):
+                             chunks=None, overwrite=True, remove_chunks=False, query_type='positional'):
         """
         Load photometric data from the IRSA server for the matched sample. The result will be saved under
 
@@ -551,10 +493,12 @@ class WISEDataBase(abc.ABC):
         :type wait: float
         :param chunks: containing indices of chunks to download
         :type chunks: list-like
+        :param query_type: 'positional': query photometry based on distance from object, 'by_allwise_id': select all photometry points within a radius of 50 arcsec with the corresponding AllWISE ID
+        :type query_type: str
         """
 
         mag = True
-        flux = False
+        flux = True
 
         if tables is None:
             tables = [
@@ -562,6 +506,9 @@ class WISEDataBase(abc.ABC):
                 'NEOWISE-R Single Exposure (L1b) Source Table'
             ]
 
+        if query_type not in self.query_types:
+            raise ValueError(f"Unknown query type {query_type}! Choose one of {self.query_types}")
+
         if chunks is None:
             chunks = list(range(round(int(self.n_chunks * perc))))
 
@@ -569,12 +516,15 @@ class WISEDataBase(abc.ABC):
             elements_per_chunk = len(self.parent_sample.df) / self.n_chunks
             service = 'tap' if elements_per_chunk > 300 else 'gator'
 
+        if (query_type == 'by_allwise_id') and (service == 'gator'):
+            raise ValueError(f"Query type 'by_allwise_id' only implemented for service 'tap'!")
+
         logger.debug(f"Getting {perc * 100:.2f}% of lightcurve chunks ({len(chunks)}) via {service} "
                      f"in {'magnitude' if mag else ''} {'flux' if flux else ''} "
                      f"from {tables}")
 
         if service == 'tap':
-            self._query_for_photometry(tables, chunks, wait, mag, flux, nthreads)
+            self._query_for_photometry(tables, chunks, wait, mag, flux, nthreads, query_type)
 
         elif service == 'gator':
             self._query_for_photometry_gator(tables, chunks, mag, flux, nthreads)
@@ -583,8 +533,8 @@ class WISEDataBase(abc.ABC):
         for c in chunks:
             self.calculate_metadata(service=service, chunk_number=c, overwrite=True)
 
-        self._combine_lcs(service=service, overwrite=True, remove=remove_chunks)
-        self._combine_metadata(service=service, overwrite=True, remove=remove_chunks)
+        self._combine_lcs(service=service, overwrite=overwrite, remove=remove_chunks)
+        self._combine_metadata(service=service, overwrite=overwrite, remove=remove_chunks)
 
     def _lightcurve_filename(self, service, chunk_number=None, jobID=None):
         if (chunk_number is None) and (jobID is None):
@@ -613,6 +563,9 @@ class WISEDataBase(abc.ABC):
         fn = self._lightcurve_filename(service, chunk_number, jobID)
         logger.debug(f"saving {len(lcs)} new lightcurves to {fn}")
 
+        if fn == self._lightcurve_filename(service):
+            self._cached_final_products['lightcurves'][service] = lcs
+
         if not overwrite:
             try:
                 old_lcs = self._load_lightcurves(service=service, chunk_number=chunk_number, jobID=jobID)
@@ -635,7 +588,9 @@ class WISEDataBase(abc.ABC):
         :return: the binned lightcurves
         :rtype: dict
         """
-        return self._load_lightcurves(service)
+        if service not in self._cached_final_products['lightcurves']:
+            self._cached_final_products['lightcurves'][service] = self._load_lightcurves(service)
+        return self._cached_final_products['lightcurves'][service]
 
     def _combine_lcs(self, service=None, chunk_number=None, remove=False, overwrite=False):
         if not service:
@@ -714,8 +669,8 @@ class WISEDataBase(abc.ABC):
     def _gator_photometry_worker_thread(self):
         while True:
             try:
-                args = self.queue.get()
-            except AttributeError:
+                args = self.queue.get(block=False)
+            except (AttributeError, queue.Empty):
                 logger.debug('No more tasks, exiting')
                 break
             logger.debug(f"{args}")
@@ -726,7 +681,7 @@ class WISEDataBase(abc.ABC):
     def _query_for_photometry_gator(self, tables, chunks, mag, flux, nthreads):
         nthreads = min(nthreads, len(chunks))
         logger.debug(f'starting {nthreads} workers')
-        threads = [threading.Thread(target=self._gator_photometry_worker_thread, daemon=True) for _ in range(nthreads)]
+        threads = [threading.Thread(target=self._gator_photometry_worker_thread) for _ in range(nthreads)]
 
         logger.debug(f"using {len(chunks)} chunks")
         self.queue = queue.Queue()
@@ -740,6 +695,9 @@ class WISEDataBase(abc.ABC):
         self.queue.join()
         self.queue = None
 
+        for t in threads:
+            t.join()
+
     def _get_unbinned_lightcurves_gator(self, chunk_number, clear=False):
         # load only the files for this chunk
         fns = [os.path.join(self._cache_photometry_dir, fn)
@@ -756,7 +714,8 @@ class WISEDataBase(abc.ABC):
 
             t = 'allwise_p3as_mep' if 'allwise' in fn else 'neowiser_p1bs_psd'
             nice_name = self.get_db_name(t, nice=True)
-            cols = dict(self.photometry_table_keymap[nice_name]['mag'])
+            cols = {'index_01': self._tap_orig_id_key}
+            cols.update(self.photometry_table_keymap[nice_name]['mag'])
             cols.update(self.photometry_table_keymap[nice_name]['flux'])
             if 'allwise' in fn:
                 cols['cntr_mf'] = 'allwise_cntr'
@@ -770,40 +729,6 @@ class WISEDataBase(abc.ABC):
         lightcurves = pd.concat(_data)
         return lightcurves
 
-    def _subprocess_select_and_bin_gator(self, chunk_number=None, jobID=None):
-        binned_lcs = dict()
-        lightcurves = self._get_unbinned_lightcurves_gator(chunk_number,
-                                                           clear=self.clear_unbinned_photometry_when_binning)
-
-        if jobID:
-            _indices = np.where(self.cluster_jobID_map == jobID)[0]
-
-        else:
-            _indices = lightcurves['index_01'].unique()
-
-        for parent_sample_idx in _indices:
-            parent_sample_idx_mask = lightcurves['index_01'] == parent_sample_idx
-            selected_data = lightcurves[parent_sample_idx_mask]
-
-            lum_keys = [c for c in lightcurves.columns if ("W1" in c) or ("W2" in c)]
-            lightcurve = selected_data[['mjd'] + lum_keys]
-            binned_lc = self.bin_lightcurve(lightcurve)
-            binned_lc = self.add_flux_density(
-                binned_lc,
-                mag_key=f'{self.mean_key}{self.mag_key_ext}',
-                emag_key=f'{self.mag_key_ext}{self.rms_key}',
-                mag_ul_key=f'{self.mag_key_ext}{self.upper_limit_key}',
-                f_key=f'{self.mean_key}{self.flux_density_key_ext}',
-                ef_key=f'{self.flux_density_key_ext}{self.rms_key}',
-                f_ul_key=f'{self.flux_density_key_ext}{self.upper_limit_key}'
-            )
-
-
-            binned_lcs[f"{int(parent_sample_idx)}"] = binned_lc.to_dict()
-
-        logger.debug(f"chunk {chunk_number}: saving {len(binned_lcs.keys())} binned lcs")
-        self._save_lightcurves(binned_lcs, service='gator', chunk_number=chunk_number, jobID=jobID, overwrite=True)
-
     # ------------------------------------------ #
     # END using GATOR to get photometry          #
     # ----------------------------------------------------------------------------------- #
@@ -812,7 +737,7 @@ class WISEDataBase(abc.ABC):
     # START using TAP to get photometry        #
     # ---------------------------------------- #
 
-    def _get_photometry_query_string(self, table_name, mag, flux):
+    def _get_photometry_query_string(self, table_name, mag, flux, query_type):
         """
         Construct a query string to submit to IRSA
         :param table_name: str, table name
@@ -822,61 +747,78 @@ class WISEDataBase(abc.ABC):
         logger.debug(f"constructing query for {table_name}")
         db_name = self.get_db_name(table_name)
         nice_name = self.get_db_name(table_name, nice=True)
+        id_key = 'cntr_mf' if 'allwise' in db_name else 'allwise_cntr'
         lum_keys = list()
         if mag:
             lum_keys += list(self.photometry_table_keymap[nice_name]['mag'].keys())
         if flux:
             lum_keys += list(self.photometry_table_keymap[nice_name]['flux'].keys())
-        keys = ['mjd'] + lum_keys
-        id_key = 'cntr_mf' if 'allwise' in db_name else 'allwise_cntr'
+        keys = ['ra', 'dec', 'mjd', id_key] + lum_keys
+        _constraints = list(self.constraints)
 
         q = 'SELECT \n\t'
         for k in keys:
             q += f'{db_name}.{k}, '
         q += f'\n\tmine.{self._tap_orig_id_key} \n'
         q += f'FROM\n\tTAP_UPLOAD.ids AS mine \n'
-        q += f'RIGHT JOIN\n\t{db_name} \n'
+
+        if query_type == 'positional':
+            q += f'RIGHT JOIN\n\t{db_name} \n'
+            radius = self.min_sep
+
+        if query_type == 'by_allwise_id':
+            q += f'INNER JOIN\n\t{db_name} ON {db_name}.{id_key} = mine.{self._tap_wise_id_key} \n'
+            radius = 15 * u.arcsec
+
         q += 'WHERE \n'
-        q += f"\tCONTAINS(POINT('J2000',{db_name}.ra,{db_name}.dec)," \
-             f"CIRCLE('J2000',mine.ra_in,mine.dec_in,0.00083))=1 "
-        if len(self.constraints) > 0:
-            q += ' AND (\n'
-            for c in self.constraints:
+
+        if query_type == 'positional':
+            q += f"\tCONTAINS(POINT('J2000',{db_name}.ra,{db_name}.dec)," \
+                 f"CIRCLE('J2000',mine.ra_in,mine.dec_in,{radius.to('deg').value}))=1 "
+
+        if len(_constraints) > 0:
+
+            if query_type == 'positional':
+                q += ' AND (\n'
+
+            for c in _constraints:
                 q += f'\t{db_name}.{c} AND \n'
             q = q.strip(" AND \n")
-            q += '\t)'
+
+            if query_type == 'positional':
+                q += '\t)'
 
         logger.debug(f"\n{q}")
         return q
 
-    def _submit_job_to_TAP(self, chunk_number, table_name, mag, flux):
+    def _submit_job_to_TAP(self, chunk_number, table_name, mag, flux, query_type):
         i = chunk_number
         t = table_name
         m = self.chunk_map == i
 
         # if perc is smaller than one select only a subset of wise IDs
         sel = self.parent_sample.df[np.array(m)]
-        # wise_id_sel = np.array(sel[self.parent_wise_source_id_key]).astype(int)
-        id_sel = np.array(sel.index).astype(int)
-        ra_sel = np.array(sel[self.parent_sample.default_keymap['ra']]).astype(float)
-        dec_sel = np.array(sel[self.parent_sample.default_keymap['dec']]).astype(float)
+
+        tab_d = dict()
+
+        tab_d[self._tap_orig_id_key] = np.array(sel.index).astype(int)
+        tab_d['ra_in'] = np.array(sel[self.parent_sample.default_keymap['ra']]).astype(float)
+        tab_d['dec_in'] = np.array(sel[self.parent_sample.default_keymap['dec']]).astype(float)
+
+        if query_type == 'by_allwise_id':
+            tab_d[self._tap_wise_id_key] = np.array(sel[self.parent_wise_source_id_key]).astype(int)
+
         del sel
 
-        upload_table = Table({
-            # self._tap_wise_id_key: wise_id_sel,
-            self._tap_orig_id_key: id_sel,
-            'ra_in': ra_sel,
-            'dec_in': dec_sel
-        })
-        logger.debug(f"{chunk_number}th query of {table_name}: uploading {len(upload_table)} objects.")
-        qstring = self._get_photometry_query_string(t, mag, flux)
+        logger.debug(f"{chunk_number}th query of {table_name}: uploading {len(list(tab_d.values())[0])} objects.")
+        qstring = self._get_photometry_query_string(t, mag, flux, query_type)
 
         N_tries = 5
         while True:
             if N_tries == 0:
                 logger.warning("No more tries left!")
             try:
-                job = WISEDataBase.service.submit_job(qstring, uploads={'ids': upload_table})
+                job = WISEDataBase.service.submit_job(qstring, uploads={'ids': Table(tab_d)})
                 job.run()
                 logger.info(f'submitted job for {t} for chunk {i}: ')
                 logger.debug(f'Job: {job.url}; {job.phase}')
@@ -923,6 +865,8 @@ class WISEDataBase(abc.ABC):
         logger.debug(f"{i}th query of {t}: saving under {fn}")
         cols = dict(self.photometry_table_keymap[t]['mag'])
         cols.update(self.photometry_table_keymap[t]['flux'])
+        if 'allwise' in t:
+            cols['cntr_mf'] = 'allwise_cntr'
         lightcurve.rename(columns=cols).to_csv(fn)
         return
 
@@ -967,12 +911,12 @@ class WISEDataBase(abc.ABC):
             else:
                 logger.warning(f'queue {i} of {t}: Job not active! Phase is {phase}')
 
-            time.sleep(np.random.uniform(600))
+            time.sleep(np.random.uniform(60))
 
         logger.debug("closing thread")
 
     def _run_tap_worker_threads(self, nthreads):
-        threads = [threading.Thread(target=self._tap_photometry_worker_thread, daemon=True)
+        threads = [threading.Thread(target=self._tap_photometry_worker_thread)
                    for _ in range(nthreads)]
 
         for t in threads:
@@ -987,10 +931,12 @@ class WISEDataBase(abc.ABC):
         for i, t in enumerate(threads):
             logger.debug(f"{i}th thread alive: {t.is_alive()}")
 
+        for t in threads:
+            t.join()
         self.tap_jobs = None
         del threads
 
-    def _query_for_photometry(self, tables, chunks, wait, mag, flux, nthreads):
+    def _query_for_photometry(self, tables, chunks, wait, mag, flux, nthreads, query_type):
         # ----------------------------------------------------------------------
         #     Do the query
         # ----------------------------------------------------------------------
@@ -1001,7 +947,7 @@ class WISEDataBase(abc.ABC):
         for t in tables:
             self.tap_jobs[t] = dict()
             for i in chunks:
-                self._submit_job_to_TAP(i, t, mag, flux)
+                self._submit_job_to_TAP(i, t, mag, flux, query_type)
                 logger.info("waiting for 2 minutes")
                 time.sleep(2*60)
 
@@ -1020,14 +966,9 @@ class WISEDataBase(abc.ABC):
         logger.info('selecting individual lightcurves and bin ...')
         ncpu = min(self.n_chunks, ncpu)
         logger.debug(f"using {ncpu} CPUs")
-        args = list(range(self.n_chunks)) if not chunks else chunks
-        logger.debug(f"multiprocessing arguments: {args}")
-
-        _fcts = {
-            'gator': self._subprocess_select_and_bin_gator,
-            'tap': self._subprocess_select_and_bin
-        }
-        fct = _fcts[service]
+        chunk_list = list(range(self.n_chunks)) if not chunks else chunks
+        service_list = [service] * len(chunk_list)
+        logger.debug(f"multiprocessing arguments: chunks: {chunk_list}, service: {service_list}")
 
         while True:
             try:
@@ -1041,13 +982,20 @@ class WISEDataBase(abc.ABC):
                 ncpu = int(round(ncpu - 1))
 
         if ncpu > 1:
-            r = list(tqdm.tqdm(
-                p.imap(fct, args), total=self.n_chunks, desc='select and bin'
-            ))
+            r = list(
+                tqdm.tqdm(
+                    p.starmap(
+                        self._subprocess_select_and_bin,
+                        zip(service_list, chunk_list)
+                    ),
+                    total=self.n_chunks,
+                    desc='select and bin'
+                )
+            )
             p.close()
             p.join()
         else:
-            r = list(map(fct, args))
+            r = list(map(self._subprocess_select_and_bin, service_list, chunk_list))
 
     def _get_unbinned_lightcurves(self, chunk_number, clear=False):
         # load only the files for this chunk
@@ -1065,12 +1013,22 @@ class WISEDataBase(abc.ABC):
 
         return lightcurves
 
-    def _subprocess_select_and_bin(self, chunk_number=None, jobID=None):
+    def _subprocess_select_and_bin(self, service, chunk_number=None, jobID=None):
         # run through the ids and bin the lightcurves
-        lightcurves = self._get_unbinned_lightcurves(chunk_number, clear=self.clear_unbinned_photometry_when_binning)
+        if service == 'tap':
+            lightcurves = self._get_unbinned_lightcurves(
+                chunk_number,
+                clear=self.clear_unbinned_photometry_when_binning
+            )
+        elif service == 'gator':
+            lightcurves = self._get_unbinned_lightcurves_gator(
+                chunk_number,
+                clear=self.clear_unbinned_photometry_when_binning
+            )
+        else:
+            raise ValueError(f"Service {service} not known!")
 
         if jobID:
-            # _chunk_number = self.clusterJob_chunk_map.loc[jobID, 'chunk_number']
             indices = np.where(self.cluster_jobID_map == jobID)[0]
         else:
             indices = lightcurves[self._tap_orig_id_key].unique()
@@ -1086,21 +1044,11 @@ class WISEDataBase(abc.ABC):
                 logger.warning(f"No data for {parent_sample_entry_id}")
                 continue
 
-            # ID = lightcurve[self._tap_wise_id_key].iloc[0]
             binned_lc = self.bin_lightcurve(lightcurve)
-            binned_lc = self.add_flux_density(
-                binned_lc,
-                mag_key=f'{self.mean_key}{self.mag_key_ext}',
-                emag_key=f'{self.mag_key_ext}{self.rms_key}',
-                mag_ul_key=f'{self.mag_key_ext}{self.upper_limit_key}',
-                f_key=f'{self.mean_key}{self.flux_density_key_ext}',
-                ef_key=f'{self.flux_density_key_ext}{self.rms_key}',
-                f_ul_key=f'{self.flux_density_key_ext}{self.upper_limit_key}'
-            )
             binned_lcs[f"{int(parent_sample_entry_id)}"] = binned_lc.to_dict()
 
         logger.debug(f"chunk {chunk_number}: saving {len(binned_lcs.keys())} binned lcs")
-        self._save_lightcurves(binned_lcs, service='tap', chunk_number=chunk_number, jobID=jobID, overwrite=True)
+        self._save_lightcurves(binned_lcs, service=service, chunk_number=chunk_number, jobID=jobID, overwrite=True)
 
     # ---------------------------------------- #
     # END using TAP to get photometry          #
@@ -1402,11 +1350,11 @@ class WISEDataBase(abc.ABC):
 
             if service == 'tap':
                 unbinned_lcs = self._get_unbinned_lightcurves(_chunk_number)
-                unbinned_lc = unbinned_lcs[unbinned_lcs[self._tap_orig_id_key] == int(parent_sample_idx)]
 
             else:
                 unbinned_lcs = self._get_unbinned_lightcurves_gator(_chunk_number)
-                unbinned_lc = unbinned_lcs[unbinned_lcs.index_01 == int(parent_sample_idx)]
+
+            unbinned_lc = unbinned_lcs[unbinned_lcs[self._tap_orig_id_key] == int(parent_sample_idx)]
 
         else:
             unbinned_lc = None
@@ -1496,6 +1444,9 @@ class WISEDataBase(abc.ABC):
     def _save_metadata(self, metadata, service, chunk_number=None, jobID=None, overwrite=False):
         fn = self._metadata_filename(service, chunk_number, jobID)
 
+        if fn == self._metadata_filename(service):
+            self._cached_final_products['metadata'][service] = metadata
+
         if not overwrite:
             try:
                 old_metadata = self._load_metadata(service=service, chunk_number=chunk_number, jobID=jobID)
@@ -1516,7 +1467,9 @@ class WISEDataBase(abc.ABC):
         :return: the metadata
         :rtype: dict
         """
-        return self._load_metadata(service)
+        if not service in self._cached_final_products['metadata']:
+            self._cached_final_products['metadata'][service] = self._load_metadata(service)
+        return self._cached_final_products['metadata'][service]
 
     def calculate_metadata(self, service, chunk_number=None, jobID=None, overwrite=True):
         """Calculates the metadata for all downloaded lightcurves.
